"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

exports.__esModule = true;
exports.getColorPrefix = getColorPrefix;
exports.createColorRegexp = createColorRegexp;
exports.removeColor = removeColor;
exports.run = run;
exports.singleProcess = singleProcess;
exports.singleCall = singleCall;
exports.runOnce = void 0;

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _regenerator = _interopRequireDefault(require("@babel/runtime-corejs3/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/asyncToGenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime-corejs3/helpers/extends"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _some = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/some"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _setTimeout2 = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set-timeout"));

var _now = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/date/now"));

var _indexOf = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/index-of"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _child_process = require("child_process");

var _kleur = _interopRequireDefault(require("kleur"));

var _psTree = _interopRequireDefault(require("ps-tree"));

var _readline = _interopRequireDefault(require("readline"));

var _stream = require("stream");

var _contracts = require("./contracts");

var _globalConfig = require("./globalConfig");

// eslint-disable-next-line max-len

/* eslint-disable no-unused-vars,callback-return,no-process-exit,no-process-env,no-extra-semi,@typescript-eslint/no-extra-semi */
// import spawn from 'spawn-command-with-kill'
// import kill from 'tree-kill'
// region helpers
function escapeRegExp(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&'); // $& means the whole matched string
} // region color


function getColorPrefix(colorFunc) {
  var colorText = colorFunc('COLOR');
  return colorText.match(/^([\s\S]*)COLOR/)[1];
} // eslint-disable-next-line @typescript-eslint/no-shadow


function createColorRegexp(colors) {
  var _context;

  return new RegExp(`[^\\r\\n]*(${(0, _map.default)(_context = (0, _map.default)(colors).call(colors, getColorPrefix)).call(_context, escapeRegExp).join('|')})[^\\r\\n]*`);
}

function removeColor(message) {
  // eslint-disable-next-line no-control-regex
  return message.replace(/\u001B\[\d+m/g, '');
} // endregion
// endregion
// region output handlers
// region stdOutSearchError
// const errorTextRegExp = /\b(err(ors?)?|warn(ings?)?|fail(ed|ure|s)?)\b|[✗]/i


var errorTextRegExp = /[^\r\n]*(\b[1-9]\d* *(fail|err)|[✗×]|fatal error|error occur)[^\r\n]*/i;
var errorColorRegExp = createColorRegexp([_kleur.default.bold, _kleur.default.red, _kleur.default.magenta, // colors.yellow,
_kleur.default.bgRed, _kleur.default.bgMagenta // colors.bgYellow,
]);

function stdOutSearchError(text) {
  return (0, _globalConfig.getGlobalConfig)().stdOutSearchError ? (0, _globalConfig.getGlobalConfig)().stdOutSearchError(text, _stdOutSearchError) : _stdOutSearchError(text);
}

function _stdOutSearchError(text) {
  var errorColor = text.match(errorColorRegExp);
  text = removeColor(text);

  if (errorColor // at least 10 letters
  && /([0-9A-Z_a-z][\0-\/:-@\[-\^`\{-\uFFFF]*){10,}/.test(text) && !/\u221A/.test(text) // electron-builder
  && !/[\u2502\u250C]/.test(text) // sapper: "189 kB client.905ef984.js"
  && !(/\b\d+\s+\w+\s+\S+\.js\b/.test(text) && text.length < 100)) {
    return `ERROR COLOR: ${errorColor[0]}`;
  }

  var errorText = text.match(errorTextRegExp);

  if (errorText) {
    return `ERROR TEXT: ${errorText[0]}`;
  }

  return false;
} // endregion
// region stdErrIsError


function correctLog(message) {
  message = message.replace(/^\s{20,}/, '');
  return message;
}

function stdErrIsError(text) {
  return (0, _globalConfig.getGlobalConfig)().stdErrIsError ? (0, _globalConfig.getGlobalConfig)().stdErrIsError(text, _stdErrIsError) : _stdErrIsError(text);
}

function _stdErrIsError(text) {
  text = removeColor(text);

  if (text.length < 20) {
    return false;
  }

  if (/openssl config failed/.test(text)) {
    return false;
  } // web storm


  if (/Debugger attached|Debugger listening on|Waiting for the debugger|nodejs.*inspector/.test(text)) {
    return false;
  } // rollup


  if (/treating it as an external dependency|\bcreated\b.*\.js in \d|\bFinished in\b/.test(text)) {
    return false;
  }

  if ((0, _indexOf.default)(text).call(text, '→') >= 0) {
    return false;
  } // someone package is outdated


  if (/\bnpm update\b/.test(text)) {
    return false;
  } // terminate process


  if (/^\^[A-Z]$/.test(text)) {
    return false;
  } // experimental warnings


  if (/ExperimentalWarning: Conditional exports is an experimental feature. This feature could change at any time/.test(text)) {
    return false;
  } // Entry module "rollup.config.js" is implicitly using "default" export mode,
  // which means for CommonJS outputthat its default export is assigned to "module.exports".
  // For many tools, such CommonJS output will not be interchangeable with the original ES module.
  // If this is intended, explicitly set "output.exports" to either "auto" or "default",
  // otherwise you might want to consider changing the signature of "rollup.config.js"
  // to use named exports only.


  if (/explicitly set "output.exports" to either "auto" or "default"/.test(text)) {
    return false;
  }

  return true;
} // endregion
// region logFilter


function logFilter(text) {
  return (0, _globalConfig.getGlobalConfig)().logFilter ? (0, _globalConfig.getGlobalConfig)().logFilter(text, _logFilter) : _logFilter(text);
}

function _logFilter(text) {
  text = removeColor(text); // sapper export

  if (/\s{4,}\S\s[^\w\r\n]*node_modules/.test(text)) {
    return false;
  } // Empty space


  if (/^[\t-\r \xA0\u1680\u2000-\u200A\u2028\u2029\u202F\u205F\u3000\uFEFF]*$/.test(text)) {
    return false;
  }

  return true;
} // endregion
// endregion
// interface ProcessEnvOptions {
// 	uid?: number;
// 	gid?: number;
// 	cwd?: string;
// 	env?: NodeJS.ProcessEnv;
// 	/**
// 	 * @default true
// 	 */
// 	windowsHide?: boolean;
// 	/**
// 	 * @default 0
// 	 */
// 	timeout?: number;
// 	argv0?: string;
// 	stdio?: StdioOptions;
// 	detached?: boolean;
// 	shell?: boolean | string;
// 	windowsVerbatimArguments?: boolean;
// 	stdio?: 'pipe' | Array<null | undefined | 'pipe'>;
// }


var wasKillAll;
var processList = [];
var runStates = [];
process.on('SIGTERM', function () {
  console.log('SIGTERM');
  killAll(true);
});
process.on('SIGHUP', function () {
  console.log('SIGHUP');
  killAll(true);
});
process.on('SIGINT', function () {
  console.log('SIGINT');
  killAll(true);
});
process.on('SIGBREAK', function () {
  console.log('SIGBREAK');
  killAll(true);
});
process.on('beforeExit', function () {
  // console.log('beforeExit')
  killAll();
});
process.on('exit', function () {
  // console.log('exit')
  killAll();
}); // process.on('disconnect', killAll)

process.on('uncaughtException', function (err) {
  printError('uncaughtException', err);
  killAll(true);
});

function printRunStates() {
  for (var i = 0; i < runStates.length; i++) {
    var state = runStates[i];
    var message = `${state.status} (${((state.timeEnd || (0, _now.default)()) - state.timeStart) / 1000} sec): ${state.command}`;

    switch (state.status) {
      case _contracts.RunStatus.RUNNED:
        console.log(_kleur.default.blue(message));
        break;

      case _contracts.RunStatus.SUCCESS:
        console.log(_kleur.default.cyan(message));
        break;

      case _contracts.RunStatus.ERROR:
        console.error(_kleur.default.red(message));
        break;

      default:
        throw new Error(`Unknown status: ${state.status}`);
    }
  }
}

function printError(prefix, err) {
  console.error(_kleur.default.red().bold(`${prefix}: ${err && err.stack || err && err.toString() || err}`));
}

function addProcess(proc) {
  processList.push(proc);
}

function _killByPidsUnix() {
  for (var _len = arguments.length, pids = new Array(_len), _key = 0; _key < _len; _key++) {
    pids[_key] = arguments[_key];
  }

  if (!pids.length) {
    return;
  }

  var params = (0, _map.default)(pids).call(pids, function (o) {
    return o.toString();
  });
  params.unshift('-15');
  console.log(`kill ${params.join(' ')}`);
  (0, _child_process.spawn)('kill', params, {
    detached: true,
    stdio: 'ignore'
  }) // .on('error', err => printError('kill error', err))
  .unref();
}

function killByPidsUnix() {
  if (!arguments.length) {
    return;
  }

  _killByPidsUnix(...arguments);

  for (var i = 0; i < arguments.length; i++) {
    (0, _psTree.default)(i < 0 || arguments.length <= i ? undefined : arguments[i], function (err, children) {
      if (err) {
        printError('psTree error', err);
        children = [];
      }

      _killByPidsUnix(...(0, _map.default)(children).call(children, function (o) {
        return o.PID;
      }));
    });
  }
}

function killByPidsWindows() {
  for (var _len2 = arguments.length, pids = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    pids[_key2] = arguments[_key2];
  }

  if (!pids.length) {
    return;
  }

  var params = ['/F', '/T'];

  for (var i = 0; i < pids.length; i++) {
    params.push('/PID');
    params.push(pids[i].toString());
  }

  console.log(`taskkill ${params.join(' ')}`);
  (0, _child_process.spawn)('taskkill', params, {
    detached: true,
    stdio: 'ignore'
  }) // .on('error', err => printError('kill error', err))
  .unref();
}

function killByPids() {
  if (arguments.length) {
    // console.log(`Kill All: ${pids.join(' ')}`)
    if (process.platform === 'win32') {
      killByPidsWindows(...arguments);
    } else {
      killByPidsUnix(...arguments);
    }
  }
}

function killAll(isFailure) {
  if (wasKillAll) {
    return;
  }

  wasKillAll = true;
  console.log('Terminating...');
  (0, _setTimeout2.default)(function () {
    var procs = (0, _filter.default)(processList).call(processList, function (o) {
      return o.pid && !o.killed;
    });
    var pids = (0, _map.default)(procs).call(procs, function (o) {
      return o.pid;
    });
    killByPids(...pids);
    printRunStates();

    if (isFailure || (0, _some.default)(runStates).call(runStates, function (o) {
      return o.status === _contracts.RunStatus.ERROR;
    })) {
      process.exit(1);
    }
  }, 100);
} // Buffer class


function run(command, _temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      env = _ref.env,
      timeout = _ref.timeout,
      notAutoKill = _ref.notAutoKill,
      stdin = _ref.stdin,
      _ref$shell = _ref.shell,
      shell = _ref$shell === void 0 ? true : _ref$shell,
      prepareProcess = _ref.prepareProcess;

  return new _promise.default(function (resolve, reject) {
    if (wasKillAll) {
      reject('Was kill all');
      return;
    }

    console.log(_kleur.default.blue(`RUN: ${command}`));
    var runState = {
      status: _contracts.RunStatus.RUNNED,
      timeStart: (0, _now.default)(),
      timeEnd: void 0,
      command
    };
    runStates.push(runState);

    var _resolve = function _resolve(value) {
      runState.status = _contracts.RunStatus.SUCCESS;
      runState.timeEnd = (0, _now.default)();
      resolve(value);
    };

    var _reject = function _reject(err) {
      runState.status = _contracts.RunStatus.ERROR;
      runState.timeEnd = (0, _now.default)();
      reject(err);
    };

    var proc = (0, _child_process.spawn)(command, {
      cwd: process.cwd(),
      env: (0, _extends2.default)({}, process.env, env),
      timeout,
      stdio: [stdin, 'pipe', 'pipe'],
      shell
    });
    var stdoutString = '';
    proc.stdout.pipe(new _stream.Writable({
      write(chunk, encoding, callback) {
        stdoutString += chunk.toString(encoding === 'buffer' ? void 0 : encoding);
      }

    }));

    if (!notAutoKill) {
      addProcess(proc);
    }

    proc.on('disconnect', function () {
      _reject('process.disconnect');
    }).on('close', function (code, signal) {
      if (code) {
        _reject(`process.close(code=${code}, signal=${signal})`);
      } else {
        _resolve(stdoutString);
      }
    }).on('exit', function (code, signal) {
      if (code) {
        _reject(`process.exit(code=${code}, signal=${signal})`);
      } else {
        _resolve(stdoutString);
      }
    }).on('message', function (message) {
      console.log(`process.message: ${message}`);
    }).on('error', function (err) {
      _reject(err);
    });

    if (proc.stdout) {
      _readline.default.createInterface({
        input: proc.stdout,
        terminal: false
      }).on('line', function (line) {
        try {
          var _error = stdOutSearchError(line);

          if (logFilter(line)) {
            line = correctLog(line);
            process.stdout.write(`${line}\r\n`);
          }

          if (_error) {
            _reject(`ERROR DETECTED: ${_error}`);
          }
        } catch (ex) {
          _reject(ex);
        }
      });
    }

    if (proc.stderr) {
      _readline.default.createInterface({
        input: proc.stderr,
        terminal: false
      }).on('line', function (line) {
        try {
          if (stdErrIsError(line)) {
            process.stdout.write(`STDERR: ${line}\r\n`);

            _reject(line);

            return;
          }

          if (logFilter(line)) {
            line = correctLog(line);
            process.stdout.write(`${line}\r\n`);
          }
        } catch (ex) {
          _reject(ex);
        }
      });
    }

    if (prepareProcess) {
      prepareProcess(proc);
    }
  }).catch(function (err) {
    if (!wasKillAll) {
      console.error(_kleur.default.bold().red(`✗ ${command}\r\n${err && err.stack || err && err.toString() || err}`));
      return _promise.default.reject(err);
    }

    return null;
  });
}

var runOnce = singleCall(run) // eslint-disable-next-line no-extend-native
;
exports.runOnce = runOnce;

_promise.default.prototype.stopOnError = function stopOnError() {
  return this.catch(function (err) {
    printError('Kill on error', err);
    killAll(true);
  });
};

function singleProcess(func) {
  var locker;
  return /*#__PURE__*/(0, _asyncToGenerator2.default)( /*#__PURE__*/_regenerator.default.mark(function _callee() {
    var _args = arguments;
    return _regenerator.default.wrap(function _callee$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return locker;

          case 2:
            locker = func(..._args);
            return _context2.abrupt("return", locker);

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee);
  }));
}

function singleCall(func) {
  var cache = {};
  return function () {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var id = (0, _stringify.default)(args);
    var cacheItem = cache[id];

    if (cacheItem) {
      if (cacheItem.error) {
        throw cacheItem.error;
      }

      return cacheItem.result;
    }

    if (cacheItem === false) {
      throw new Error(`Recursive call of single call func: ${func.toString()}`);
    }

    cache[id] = false;

    try {
      var result = func(...args);
      cache[id] = {
        result
      };
      return result;
    } catch (error) {
      cache[id] = {
        error
      };
      throw error;
    }
  };
}